#!/usr/bin/perl

#
# copyright (C) vipros@gmail.com @ http://www.vipros.com/photo
# ExifTool is copyrighted under Phil Harvey (http://www.sno.phy.queensu.ca/~phil/exiftool)
#
# 0.4		2006-07-12	Supported TimeZone for GPX
# 0.3		2005-10-15	Supported GPX tracklog format
# 0.2		2005-10-09	Improved Accuracy Algorithm
# 0.1		2005-10-05	Initial Version
#

require 5.004;
require Image::ExifTool;

$numArgs = $#ARGV + 1;
if ( $numArgs != 1 ) {
	print "Please spcify the time zone. For example,\nperl gps2Photo.pl -5\nYou are running the program in EST.";
	exit;
}
$tz = $ARGV[0];

my $exeDir;
BEGIN {
    # get exe directory
    $exeDir = $0;           # get exe directory from command
    $exeDir =~ tr/\\/\//;   # for systems that use backslashes
    # isolate directory specification
    $exeDir =~ s/(.*)\/.*/$1/ or $exeDir = '.';
    # add lib directory at start of include path
    unshift @INC, "$exeDir/lib";
}

END {
#    Cleanup();
}

use HTTP::Date;
use Time::Local;
use Image::ExifTool;

my $generatedDir = "./GPS";
my $txt = "./gps.txt";
my $gpx = "./gps.gpx";
my $gpsDataFile = $txt;

my $gps2PhotoLog = "$generatedDir/gps2Photo.log.txt";
my $exifTool = new Image::ExifTool;

if (! -d $generatedDir ) {
	mkdir $generatedDir or die "Can't create folder $generatedDir: $!. Please remove $generatedDir. Abort!\n";
} else { # remove $generatedDir
	print "Please remove folder $generatedDir.\n";
	exit;
}

if ( ! -f $gpsDataFile ) {
	$gpsDataFile = $gpx;
} 

open FH, $gpsDataFile or die "Can't find file $gpsDataFile: $!\n";

open LOG, ">>$gps2PhotoLog" or die "Can't open $gps2PhotoLog: $!\n";

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);

if ( $year >= 118 ) {
	print "gps2Photo has expired. Please visit http://www.vipros.com/photo to obtain a new copy. Thank you!";			
	exit;
}

my $logString = "\n== $gpsDataFile, Process time: %4d-%02d-%02d %02d:%02d:%02d ==\n", $year+1900,$mon+1,$mday,$hour,$min,$sec;
printf LOG $logString;
printf $logString;



# Loop current dir, looking for images
opendir MYDIR, ".";
while($file = readdir(MYDIR)) {

	next if ($file !~ /^[^GPS_].*\.[j|J][p|P][g|G]$/);

	seek(FH,0,0) or die "Failed to rewind to the beginning of the file";
	my $info = $exifTool->ImageInfo($file);
	my $photoTime = $info->{"DateTimeOriginal"};
#print "photoTime: $photoTime\n";
	$logString = "\n$file ($photoTime) ";
	printf LOG $logString;
	printf $logString;
	
	my $year;
	my $month;
	my $day;
	my $hour;
	my $minute;
	my $second;
	
	($year,$month,$day, $hour,$minute, $second)=($photoTime =~ /^\s*(\d+):(\d+):(\d+)\s+(\d+):(\d+):(\d+)\s*/x);
	
	$ptime = timegm($second, $minute, $hour, $day, $month-1, $year-1900);

	$found = 0;
	
	my $loc1, $loc2, $loc3, $loc4, $loc5, $loc6, $loc7, $loc8, $gmonth, $gday, $gyear, $ghour, $gminute, $gsecond, $gapm;
	my $preloc1, $preloc2, $preloc3, $preloc4, $preloc5, $preloc6, $preloc7, $preloc8, $pregmonth, $pregday, $pregyear, $preghour, $pregminute, $pregsecond, $pregapm;
	my $delta = 0;
	my $predelta = 0;
	while (<FH>) {
		$gpsstr = $_;
		
		if ( ($gpsDataFile cmp $txt) == 0 ) { # gps.txt
			($loc1, $loc2, $loc3, $loc4, $loc5, $loc6, $loc7, $loc8, $gmonth, $gday, $gyear, $ghour, $gminute, $gsecond, $gapm)=($gpsstr =~ /^Trackpoint.*([N|s])([0-9]+) ([0-9]+)\.([0-9]+) ([W|E])([0-9]+)[\t| ]([0-9]+)\.([0-9]+)[\t| ]([0-9]+)\/([0-9]+)\/(200[0-9]) ([0-9]+):([0-9]+):([0-9]+) ([A|P])m*/);
			if ( ($loc1 cmp "") == 0) {
				($loc1, $loc2, $loc3, $loc4, $loc5, $loc6, $loc7, $loc8, $gyear, $gmonth, $gday, $ghour, $gminute, $gsecond)=($gpsstr =~ /^Trackpoint.*([N|s])([0-9]+) ([0-9]+)\.([0-9]+) ([W|E])([0-9]+)[\t| ]([0-9]+)\.([0-9]+)[\t| ](200[0-9])-([0-9]+)-([0-9]+) ([0-9]+):([0-9]+):([0-9]+) */);	
			}
		} else { # gps.gpx
			$loc1 = "N";
			$loc5 = "E";
			if ( /trkpt.*lat/ ) {
				($lat, $lon) = ($gpsstr =~ /lat=\"(.*)\"[ ]lon=\"(.*)\"/);
				if ( $lat =~ /\-/ ) {
					($lat) = ( $lat =~ /\-(.*)/);
					$loc1 = "S";
					
					#print "\n $lat SOUTH\n";
				}
				if ( $lon =~ /\-/ ) {
					($lon) = ($lon =~ /\-(.*)/);
					$loc5 = "W";
				}
#				print "lat=$lat, lon=$lon\n";
				$gtime = "";
			} #else {
				if ( /time.*time/ ) {
					($gtime) = ($gpsstr =~ /(20.*)Z/) ;
					# $gtime = 2005-09-01T15:45:56
#					print "$gpsstr\n";
#					print "GTIME: $gtime\n";
					($gyear, $gmonth, $gday, $ghour, $gminute, $gsecond) 
						= ($gtime =~ /(20[0-9][0-9])\-([0-1][0-9])\-([0-3][0-9])T([0-9][0-9])\:([0-9][0-9])\:([0-9][0-9])/) ;
			#		print "\n$gyear, $gmonth, $gday, $ghour, $gminute, $gsecond\n"
				}
			#}
		}

		if ( $gyear != "" && $gyear < 3007 ) {
			if ( $ghour < 12 && ($gapm cmp "P") == 0 ) {
				$ghour = $ghour + 12;
			}

			$gtime = timegm($gsecond, $gminute, $ghour, $gday, $gmonth - 1, $gyear - 1900) + 3600*$tz;
			$delta = abs($ptime - $gtime);
			#print "\n$gmonth, $gday, $gyear, $ghour, $gminute, $gsecond, $gapm, delta=$delta\n";
			if ( $delta < 1500 ) {
				if ( $predelta == 0 || $delta < $predelta ) {
					$predelta = $delta;
					
					$preloc1 = $loc1;
					$preloc2 = $loc2;
					$preloc3 = $loc3;
					$preloc4 = $loc4;
					$preloc5 = $loc5;
					$preloc6 = $loc6;
					$preloc7 = $loc7;
					$preloc8 = $loc8;
					$pregmonth = $gmonth;
					$pregday = $gday;
					$pregyear = $gyear;
					$preghour = $ghour;
					$pregminute = $gminute;
					$pregsecond = $gsecond;
					next; # found a closer one; continue on loop
				}
				$found = "true";
			} # within 15 minutes
			if ( $found || $predelta != 0 ) {
				# -GPSLatitudeRef=N -GPSLatitude="43 deg 44.817" -GPSLongitudeRef=W -GPSLongitude="79 deg 22.242"  1.jpg
				if ( ($gpsDataFile cmp $txt) == 0 ) { # gps.txt
					$exifTool->SetNewValue("GPSLatitudeRef", "$preloc1");
					$exifTool->SetNewValue("GPSLatitude", "$preloc2 deg $preloc3.$preloc4");
					$exifTool->SetNewValue("GPSLongitudeRef", "$preloc5");
					$exifTool->SetNewValue("GPSLongitude", "$preloc6 deg $preloc7.$preloc8");
					$exifTool->WriteInfo($file, "$generatedDir/$file");
					$logString = "taken at $loc1$loc2 $loc3.$loc4 $loc5$loc6 $loc7.$loc8 ($pregyear\-$pregmonth\-$pregday $preghour:$pregminute:$pregsecond) ($predelta sec)";
				} else { # gps.gpx
					$exifTool->SetNewValue("GPSLatitudeRef", "$preloc1");
					$exifTool->SetNewValue("GPSLatitude", "$lat");
					$exifTool->SetNewValue("GPSLongitudeRef", "$preloc5");
					$exifTool->SetNewValue("GPSLongitude", "$lon");
					$exifTool->WriteInfo($file, "$generatedDir/$file");
					$logString = "taken at $loc1$lat $loc5$lon ($pregyear\-$pregmonth\-$pregday $preghour:$pregminute:$pregsecond) ($predelta sec)";
				}
				printf LOG $logString;
				printf $logString;
				last;
			} 
		} # if gyear == 2005
	} # while FH

} # end of loop dir
closedir MYDIR;
